package main

import (
	"net/http"
	"os"
	"validator/api"
	"validator/conf"
	"validator/core"
	"validator/quest"
	"validator/version"

	"github.com/prometheus/client_golang/prometheus/promhttp"
	log "github.com/sirupsen/logrus"
	"github.com/urfave/cli/v2"
)

var (
	appFlags = []cli.Flag{
		configFileFlag,
		dataDirFlag,
		metricsListenAddrFlag,
		logLevelFlag,
		grpcListenAddrFlag,
		rpcListenAddrFlag,
		privateKeyFlag,
		chainRPCFlag,
		storeContractFlag,
		validatorContractFlag,
		commitTimeFlag,
		questHostFlag,
		questPortFlag,
		questUserFlag,
		questPassFlag,
		questDBFlag,
	}
)

func main() {
	app := cli.App{}
	app.Flags = wrapFlags(append(appFlags, []cli.Flag{}...))
	app.Name = "validator"
	app.Usage = "this is validator"
	app.Version = version.Version
	app.Before = func(c *cli.Context) error {
		return loadFlagsFromConfig(c, app.Flags)
	}
	app.Action = func(context *cli.Context) error {
		run(context)
		return nil
	}
	if err := app.Run(os.Args); err != nil {
		log.Error(err)
	}
}

func run(ctx *cli.Context) {
	cfg := &conf.Config{
		MetricsListenAddr: ctx.String(metricsListenAddrFlag.Name),
		LogLevel:          ctx.String(logLevelFlag.Name),
		GRPCListenAddr:    ctx.String(grpcListenAddrFlag.Name),
		RPCListenAddr:     ctx.String(rpcListenAddrFlag.Name),
		PrivateKey:        ctx.String(privateKeyFlag.Name),
		ChainRPC:          ctx.String(chainRPCFlag.Name),
		StoreContract:     ctx.String(storeContractFlag.Name),
		ValidatorContract: ctx.String(validatorContractFlag.Name),
		DataDir:           ctx.String(dataDirFlag.Name),
		CommitTime:        ctx.Int(commitTimeFlag.Name),
	}

	setLogLevel(cfg.LogLevel)
	runMetrics(cfg.MetricsListenAddr)

	qCfg := &conf.QuestConfig{
		Host:     ctx.String(questHostFlag.Name),
		Port:     ctx.Int(questPortFlag.Name),
		User:     ctx.String(questUserFlag.Name),
		Password: ctx.String(questPassFlag.Name),
		Database: ctx.String(questDBFlag.Name),
	}
	q := quest.NewQuest(qCfg)

	w := core.RunValidator(q, cfg)
	// runGrpcServer(cfg.GRPCListenAddr, w)
	runJSONRPCServer(cfg.RPCListenAddr, w)
	select {}
}

func setLogLevel(level string) {
	_level, err := log.ParseLevel(level)
	if err != nil {
		log.WithError(err).Error("failed to parse log level")
		return
	}
	log.WithField("log level", _level).Info("set log level")
	log.SetLevel(_level)
}

func runMetrics(listen string) {
	http.Handle("/metrics", promhttp.Handler())
	log.WithField("listen", listen).Info("start prometheus metrics server")
	go func() {
		if err := http.ListenAndServe(listen, nil); err != nil {
			log.WithError(err).Error("failed to start prometheus metrics server")
		}
	}()
}

func runJSONRPCServer(listen string, w *core.Validator) {
	go api.StartJSONRPC(listen, w)
}
